@@ -3,14 +3,12 @@
 #![allow(non_upper_case_globals)]
 #![deny(missing_copy_implementations)]
 
-extern crate libc;
-
 use libc::{
     c_char, c_int, c_long, c_short, c_uchar, c_uint, c_ulong, c_ushort, c_void, ptrdiff_t, size_t,
 };
 
 mod tt_tables;
-pub use tt_tables::*;
+pub use crate::tt_tables::*;
 
 // Basic Data Types
 pub type FT_Byte = c_uchar;
@@ -351,9 +349,29 @@ pub struct FT_SfntLangTag {
     pub string_len: FT_UInt,
 }
 
+#[repr(C)]
+#[derive(Debug, Hash, PartialEq, Eq)]
+pub struct FT_LayerIterator {
+    pub num_layers: FT_UInt,
+    pub layer: FT_UInt,
+    pub p: *mut FT_Byte,
+}
+
 // Enums
 
 pub type enum_FT_Sfnt_Tag_ = c_uint;
+pub const FT_SFNT_HEAD: u32 = 0_u32;
+pub const FT_SFNT_MAXP: u32 = 1_u32;
+pub const FT_SFNT_OS2: u32 = 2_u32;
+pub const FT_SFNT_HHEA: u32 = 3_u32;
+pub const FT_SFNT_VHEA: u32 = 4_u32;
+pub const FT_SFNT_POST: u32 = 5_u32;
+pub const FT_SFNT_PCLT: u32 = 6_u32;
+pub const FT_SFNT_MAX: u32 = 7_u32;
+pub type FT_Sfnt_Tag = enum_FT_Sfnt_Tag_;
+
+// Preserved for compatability reasons. These can be removed in the next
+// major release.
 pub const ft_sfnt_head: u32 = 0_u32;
 pub const ft_sfnt_maxp: u32 = 1_u32;
 pub const ft_sfnt_os2: u32 = 2_u32;
@@ -362,7 +380,6 @@ pub const ft_sfnt_vhea: u32 = 4_u32;
 pub const ft_sfnt_post: u32 = 5_u32;
 pub const ft_sfnt_pclt: u32 = 6_u32;
 pub const ft_sfnt_max: u32 = 7_u32;
-pub type FT_Sfnt_Tag = enum_FT_Sfnt_Tag_;
 
 pub type FT_Pixel_Mode = c_uint;
 pub const FT_PIXEL_MODE_NONE: FT_Pixel_Mode = 0;
@@ -388,7 +405,8 @@ pub const FT_RENDER_MODE_LIGHT: FT_Render_Mode = 1;
 pub const FT_RENDER_MODE_MONO: FT_Render_Mode = 2;
 pub const FT_RENDER_MODE_LCD: FT_Render_Mode = 3;
 pub const FT_RENDER_MODE_LCD_V: FT_Render_Mode = 4;
-pub const FT_RENDER_MODE_MAX: FT_Render_Mode = FT_RENDER_MODE_LCD_V + 1;
+pub const FT_RENDER_MODE_SDF: FT_Render_Mode = 5;
+pub const FT_RENDER_MODE_MAX: FT_Render_Mode = FT_RENDER_MODE_SDF + 1;
 
 pub type FT_LcdFilter = c_uint;
 pub const FT_LCD_FILTER_NONE: FT_LcdFilter = 0;
@@ -946,10 +964,6 @@ pub fn FT_HAS_COLOR(face: FT_Face) -> bool {
     unsafe { (*face).face_flags & FT_FACE_FLAG_COLOR != 0 }
 }
 
-extern "C" {
-    pub fn FT_Get_Sfnt_Table(face: FT_Face, tag: FT_Sfnt_Tag) -> *mut c_void;
-}
-
 extern "C" {
     pub fn FT_Init_FreeType(alibrary: *mut FT_Library) -> FT_Error;
     pub fn FT_Done_FreeType(library: FT_Library) -> FT_Error;
@@ -1038,7 +1052,7 @@ extern "C" {
     pub fn FT_Get_Char_Index(face: FT_Face, charcode: FT_ULong) -> FT_UInt;
     pub fn FT_Get_First_Char(face: FT_Face, agindex: *mut FT_UInt) -> FT_ULong;
     pub fn FT_Get_Next_Char(face: FT_Face, char_code: FT_ULong, agindex: *mut FT_UInt) -> FT_ULong;
-    pub fn FT_Get_Name_Index(face: FT_Face, glyph_name: *mut FT_String) -> FT_UInt;
+    pub fn FT_Get_Name_Index(face: FT_Face, glyph_name: *const c_char) -> FT_UInt;
     pub fn FT_Get_SubGlyph_Info(
         glyph: FT_GlyphSlot,
         sub_index: FT_UInt,
@@ -1140,14 +1154,7 @@ extern "C" {
         num_contours: FT_Int,
         anoutline: *mut FT_Outline,
     ) -> FT_Error;
-    pub fn FT_Outline_New_Internal(
-        memory: FT_Memory,
-        num_points: FT_UInt,
-        num_contours: FT_Int,
-        anoutline: *mut FT_Outline,
-    ) -> FT_Error;
     pub fn FT_Outline_Done(library: FT_Library, outline: *mut FT_Outline) -> FT_Error;
-    pub fn FT_Outline_Done_Internal(memory: FT_Memory, outline: *mut FT_Outline) -> FT_Error;
     pub fn FT_Outline_Copy(source: *const FT_Outline, target: *mut FT_Outline) -> FT_Error;
     pub fn FT_Outline_Translate(outline: *const FT_Outline, xOffset: FT_Pos, yOffset: FT_Pos);
     pub fn FT_Outline_Transform(outline: *const FT_Outline, matrix: *const FT_Matrix);
@@ -1185,7 +1192,7 @@ extern "C" {
     pub fn FT_GlyphSlot_Oblique(slot: FT_GlyphSlot);
 
     pub fn FT_Get_Multi_Master(face: FT_Face, amaster: *mut FT_Multi_Master) -> FT_Error;
-    pub fn FT_Get_MM_Var(face: FT_Face, amaster: *mut FT_MM_Var) -> FT_Error;
+    pub fn FT_Get_MM_Var(face: FT_Face, amaster: *mut *mut FT_MM_Var) -> FT_Error;
     pub fn FT_Done_MM_Var(library: FT_Library, amaster: *mut FT_MM_Var) -> FT_Error;
     pub fn FT_Set_MM_Design_Coordinates(
         face: FT_Face,
@@ -1247,6 +1254,15 @@ extern "C" {
         alangtag: *mut FT_SfntName,
     ) -> FT_Error;
 
+    pub fn FT_Get_Sfnt_Table(face: FT_Face, tag: FT_Sfnt_Tag) -> *mut c_void;
+    pub fn FT_Load_Sfnt_Table(
+        face: FT_Face,
+        tag: FT_ULong,
+        offset: FT_Long,
+        buffer: *mut FT_Byte,
+        length: *mut FT_ULong,
+    ) -> FT_Error;
+
     pub fn FT_Face_GetCharVariantIndex(
         face: FT_Face,
         charcode: FT_ULong,
@@ -1260,4 +1276,12 @@ extern "C" {
     pub fn FT_Face_GetVariantSelectors(face: FT_Face) -> *mut FT_UInt;
     pub fn FT_Face_GetVariantsOfChar(face: FT_Face, charcode: FT_ULong) -> *mut FT_UInt32;
     pub fn FT_Face_GetCharsOfVariant(face: FT_Face, variant_selector: FT_ULong) -> *mut FT_UInt32;
+
+    pub fn FT_Get_Color_Glyph_Layer(
+        face: FT_Face,
+        base_glyph: FT_UInt,
+        aglyph_index: *mut FT_UInt,
+        acolor_index: *mut FT_UInt,
+        iterator: *mut FT_LayerIterator,
+    ) -> bool;
 }
